var logFile, args, filename, target, shell,
    scriptPath = WScript.ScriptFullName,
    getLogFileAttempts = 50,
    exportAttempts = 5;

function eventLog(msg) {
    if (!shell) shell = WScript.CreateObject("WScript.Shell");
    shell.LogEvent(4, scriptPath + ":\n\n" + msg);
}

function objectToString(obj, name, level) {
    var str, prop, i, prefix = "    ";

    level = (level == null) ? 0 : level;

    if (level > 10) return "...";

    for (i = 0; i < level; i++) prefix += prefix;

    prefix += (name + ": ");

    if (obj === undefined) return (prefix + "undefined") + "\n";
    if (obj === null) return (prefix + "null") + "\n";
    if (typeof obj === "function") return (prefix + obj.toString()) + "\n";
    if (typeof obj === "string") return (prefix + obj) + "\n";
    if (typeof obj === "number") return (prefix + obj.toString()) + "\n";
    if (typeof obj === "boolean") return (prefix + (obj ? "true" : "false")) + "\n";

    str = prefix + "\n";
    for (prop in obj) {
        if (obj.hasOwnProperty(prop)) {
            str += objectToString(obj[prop], prop, level + 1);
        }
    }
    return str;
}

function sleep(min, max) {
    WScript.Sleep(Math.round(min + Math.random() * (max - min)) * 1000);
}

function openLogFile(fname) {
    var logFileName = WScript.ScriptFullName
            .replace(/gc-export-shape\.wsf$/, fname || "log.txt")
            .replace(/\\/g, "/");

    if (logFile) return logFile;

    try {
        logFile = fso.OpenTextFile(
                logFileName,
                8, // ForAppending
                true, // Create
                -2 // SystemDefaultFormat
            );
        return logFile;
    } catch (err) {
        eventLog(objectToString(err, "log file error (" + logFileName + ")"));
        getLogFileAttempts--;

        if (getLogFileAttempts > 0) {
            eventLog("sleeping... tries left " + getLogFileAttempts);
            sleep(10, 15);
            return openLogFile();
        }

        getLogFileAttempts = 10;
        eventLog("opening a temporary log file.");
        return openLogFile("log_" + (new Date()).getTime() + "_" + Math.round(1000 + Math.random() * 1000));
    }
}

function closeLogFile() {
    try {
        if (logFile) {
            logFile.Close();
        }
    } catch (err) {
        eventLog(objectToString(err, "error closing log file"));
    }
}

function log(msg) {
    try {
        openLogFile().WriteLine((new Date()).toUTCString() + ": " + (msg || "empty message"));
    } catch (err) {
        eventLog("error writing to log file. log msg:\n   " + (msg || "empty message"));
    }
}

function logObject(obj, name, level) {
    var prop, i, prefix = "    ";

    level = (level == null) ? 0 : level;

    if (level > 10) return;

    for (i = 0; i < level; i++) prefix += prefix;

    prefix += (name + ": ");

    if (obj === undefined) log(prefix + "undefined");
    else if (obj === null) log(prefix + "null");
    else if (typeof obj === "function") log(prefix + obj.toString());
    else if (typeof obj === "string") log(prefix + obj);
    else if (typeof obj === "number") log(prefix + obj.toString());
    else if (typeof obj === "boolean") log(prefix + (obj ? "true" : "false"));
    else {
        log(prefix);
        for (prop in obj) {
            if (obj.hasOwnProperty(prop)) {
                logObject(obj[prop], prop, level + 1);
            }
        }
    }
}

function timestampFilename(name) {
    var timestamp = Math.round((new Date()).getTime() / 1000);
    return name.replace(/\.zip$/, "_" + timestamp + ".zip");
}

function go(args) {
    var baseUrl = "https://editor.giscloud.com/datasource/export?task=exportlayer&module=layer&format=ESRI+Shapefile&name=layer_export",
        layerId = args.layer,
        apiKey = args.apikey,
        url = [baseUrl, "layerid=" + layerId, "api_key=" + apiKey].join("&"),
        xmlhttp = WScript.CreateObject("MSXML2.XMLHTTP"),
        response, stream;

    log("Starting export of layer " + layerId);

    try {
        // request shapefile export
        xmlhttp.Open("GET", url, false);
        xmlhttp.Send();
    } catch (err) {
        log("Error creating http connection, url: " + url);
        logObject(err, "error");
    }

    // check response status
    if (xmlhttp.Status !== 200) {
        log("Request failed, url " + url);
        log("Status " + xmlhttp + ", Response: " + xmlhttp.ResponseText);
        return false;
    }

    // get response text
    response = xmlhttp.ResponseText;
    if (!response || !response.length) {
        log("Empty response.");
        return false;
    }

    // fix slashes
    response = response.replace(/\\\//g, "/");

    // check result
    if (response.indexOf("\"result\":\"no\"") > -1) {
        log("Operation unsuccessful");
        return false;
    }
    if (response.indexOf("\"result\":\"yes\"") === -1) {
        log("Invalid response: ", response);
        return false;
    }

    // get exported file url
    url = response.match(/"url":"([^"]+)"/);
    if (!url || url.length !== 2) {
        log("Invalid response: ", response);
        return false;
    }
    url = url[1];

    try {
        // retrieve exported file
        xmlhttp.Open("GET", url, false);
        xmlhttp.Send();
    } catch (err) {
        log("Error retrieving exported file, url: " + url);
        logObject(err, "error");
    }

    // check response status
    if (xmlhttp.Status !== 200) {
        log("Request failed, url " + url);
        log("Status " + xmlhttp + ", Response: " + (xmlhttp.ResponseText || "no response text"));
        return false;
    }

    try {
        // open a stream to save file
        stream = WScript.CreateObject("ADODB.Stream");
    } catch (err) {
        log("Error creating ADODB stream");
        log(logObject(err, "error"));
        return false;
    }

    try {
        // write response data to stream
        stream.Open();
        stream.Type = 1; // Binary
        stream.Write(xmlhttp.ResponseBody);
        stream.Position = 0;
    } catch (err) {
        log("Error writing data to stream.");
        log(logObject(err, "error"));
        return false;
    }

    // add filename to target
    filename = url.match(/.+(\/.+)$/)[1];
    filename = timestampFilename(filename);
    target = args.dest.replace(/\\\\?/g, "/");
    target = target.replace(/\/?$/, filename);

    try {
        // create an empty target file
        if (fso.FileExists(target))
        {
            fso.DeleteFile(target);
        }
    } catch (err) {
        log("Error securing an empty file: " + target);
        log(logObject(err, "error"));
        return false;
    }

    try {
        // write to target file
        stream.SaveToFile(target, 2);
        stream.Close();
    } catch (err) {
        log("Error writing output file: " + target);
        log(logObject(err, "error"));
        return false;
    }

    // all done!
    return true;
}

function getArguments() {
    // check arguments
    if (!WScript.Arguments.Named.Exists("layer") ||
        !WScript.Arguments.Named.Exists("apikey") ||
        !WScript.Arguments.Named.Exists("dest")) {
        return false;
    }

    return {
            layer: WScript.Arguments.Named.Item("layer"),
            apikey: WScript.Arguments.Named.Item("apikey"),
            dest: WScript.Arguments.Named.Item("dest")
        };
}

try {
    // get command line arguments
    args = getArguments();
    if (!args) {
        log("Started with invalid arguments.");
        // show usage to user
        WScript.Arguments.ShowUsage();
        WScript.Quit();
    }

    // execute
    while (exportAttempts > 0) {
        exportAttempts--;
        if (go(args)) {
            log("Export successful: " + target);
            WScript.Quit();
        }

        log("Export failed. Attempts left: " + exportAttempts);
        closeLogFile();
        sleep(2, 10);
    }

    log("Export failed.");
} catch (err) {
    eventLog(objectToString(err, "error executing script"));
}